package main

import (
	"fmt"
	"os"

	tea "github.com/charmbracelet/bubbletea"
)

type commandChoice struct {
	displayString string
	commandString string
}

type model struct {
	cursor           int
	browserSessions  []string
	commands         []commandChoice
	selectedSessions map[int]struct{}
	selectedCommands map[int]struct{}
}

func getBrowserSessions() []string {
	// Placeholder
	return []string{"primary", "breakfast"}
}

func initialModel() model {
	return model{
		browserSessions: getBrowserSessions(),
		commands: []commandChoice{{
			displayString: "neomutt",
			commandString: "neomutt",
		}, {
			displayString: "bottom",
			commandString: "btm --group --battery --color gruvbox-light",
		}},
		// A map which indicates which choices are selected. We're using
		// the  map like a mathematical set. The keys refer to the indexes
		// of the `choices` slice, above.
		selectedSessions: make(map[int]struct{}),
		selectedCommands: make(map[int]struct{}),
	}
}

func (m model) Init() tea.Cmd {
	return nil
}

func (m model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
	switch msg := msg.(type) {
	case tea.KeyMsg:
		switch msg.String() {
		case "ctrl+c", "q":
			return m, tea.Quit
		case "up", "k":
			if m.cursor > 0 {
				m.cursor--
			}
		case "down", "j":
			if m.cursor < len(m.choices)-1 {
				m.cursor++
			}
		case "enter", " ":
			_, ok := m.selected[m.cursor]
			if ok {
				delete(m.selected, m.cursor)
			} else {
				m.selected[m.cursor] = struct{}{}
			}
		}
	}

	return m, nil
}

func (m model) View() string {
	s := "What should we buy at the market?\n\n"

	for i, choice := range m.choices {
		cursor := " "
		if m.cursor == i {
			cursor = ">"
		}

		checked := " "
		if _, ok := m.selected[i]; ok {
			checked = "x"
		}

		s += fmt.Sprintf("%s [%s] %s\n", cursor, checked, choice)
	}

	s += "\nPress q to quit.\n"

	return s
}

func main() {
	p := tea.NewProgram(initialModel())
	if err := p.Start(); err != nil {
		fmt.Printf("Alas, there's been an error: %v", err)
		os.Exit(1)
	}
}
